@@ -23,6 +23,7 @@ from xml.etree import ElementTree as etree
 
 # GNUmed
 if __name__ == '__main__':
+	logging.basicConfig(level = logging.DEBUG)
 	sys.path.insert(0, '../../')
 from Gnumed.pycommon import gmExceptions
 from Gnumed.pycommon import gmDispatcher
@@ -620,8 +621,15 @@ class cIdentity(gmBusinessDBObject.cBusinessDBObject):
 		active name.
 		@param nickname The preferred/nick/warrior name to set.
 		"""
-		rows, idx = gmPG2.run_rw_queries(queries = [{'cmd': u"select dem.set_nickname(%s, %s)", 'args': [self.ID, nickname]}])
-		self.refetch_payload()
+		if self._payload[self._idx['preferred']] == nickname:
+			return True
+		rows, idx = gmPG2.run_rw_queries(queries = [{'cmd': u"SELECT dem.set_nickname(%s, %s)", 'args': [self.ID, nickname]}])
+		# setting nickname doesn't change dem.identity, so other fields
+		# of dem.v_basic_person do not get changed as a consequence of
+		# setting the nickname, hence locally setting nickname matches
+		# in-database reality
+		self._payload[self._idx['preferred']] = nickname
+		#self.refetch_payload()
 		return True
 	#--------------------------------------------------------
 	def get_tags(self, order_by=None):
@@ -1732,7 +1740,7 @@ class cPatient(cIdentity):
 #		return self.__emr
 
 	def get_emr(self, allow_user_interaction=True):
-		_log.debug('accessing EMR (thread %s)', thread.get_ident())
+		_log.debug('accessing EMR for identity [%s] (thread %s)', self._payload[self._idx['pk_identity']], thread.get_ident())
 		if not self.__emr_access_lock.acquire(False):
 			got_lock = False
 			for idx in range(100):
@@ -1744,7 +1752,7 @@ class cPatient(cIdentity):
 					break
 			if not got_lock:
 				_log.error('still failed to acquire EMR access lock, aborting (thread %s)', thread.get_ident())
-				raise AttributeError('cannot lock access to EMR')
+				raise AttributeError('cannot lock access to EMR for identity [%s]', self._payload[self._idx['pk_identity']])
 
 #			# maybe something slow is happening on the machine
 #			_log.debug('failed to acquire EMR access lock, sleeping for 500ms (thread %s)', thread.get_ident())
@@ -1754,14 +1762,14 @@ class cPatient(cIdentity):
 #				raise AttributeError('cannot lock access to EMR')
 
 		if self.__emr is None:
-			_log.debug('pulling chart (thread %s)', thread.get_ident())
+			_log.debug('pulling chart for identity [%s] (thread %s)', self._payload[self._idx['pk_identity']], thread.get_ident())
 			#emr = _pull_chart(self._payload[self._idx['pk_identity']])
 			emr = _pull_chart(self)
 			if emr is None:		# user aborted pulling chart
 				return None
 			self.__emr = emr
 
-		_log.debug('returning EMR (thread %s)', thread.get_ident())
+		_log.debug('returning EMR for identity [%s] (thread %s)', self._payload[self._idx['pk_identity']], thread.get_ident())
 		self.__emr_access_lock.release()
 		return self.__emr
 
@@ -1823,7 +1831,7 @@ class gmCurrentPatient(gmBorg.cBorg):
 		"""
 		# make sure we do have a patient pointer
 		try:
-			tmp = self.patient
+			self.patient
 		except AttributeError:
 			self.patient = gmNull.cNull()
 			self.__register_interests()
@@ -1847,7 +1855,7 @@ class gmCurrentPatient(gmBorg.cBorg):
 		if patient == -1:
 			_log.debug('explicitly unsetting current patient')
 			if not self.__run_callbacks_before_switching_away_from_patient():
-				_log.debug('not unsetting current patient')
+				_log.error('not unsetting current patient, at least one pre-change callback failed')
 				return None
 			self.__send_pre_unselection_notification()
 			self.patient.cleanup()
@@ -1868,10 +1876,10 @@ class gmCurrentPatient(gmBorg.cBorg):
 			return None
 
 		# user wants different patient
-		_log.debug('patient change [%s] -> [%s] requested', self.patient['pk_identity'], patient['pk_identity'])
+		_log.info('patient change [%s] -> [%s] requested', self.patient['pk_identity'], patient['pk_identity'])
 
 		if not self.__run_callbacks_before_switching_away_from_patient():
-			_log.debug('not changing current patient')
+			_log.error('not changing current patient, at least one pre-change callback failed')
 			return None
 
 		# everything seems swell
@@ -1890,12 +1898,30 @@ class gmCurrentPatient(gmBorg.cBorg):
 		return None
 	#--------------------------------------------------------
 	def __register_interests(self):
-		gmDispatcher.connect(signal = u'dem.identity_mod_db', receiver = self._on_identity_change)
-		gmDispatcher.connect(signal = u'dem.names_mod_db', receiver = self._on_identity_change)
+		gmDispatcher.connect(signal = u'gm_table_mod', receiver = self._on_database_signal)
+
 	#--------------------------------------------------------
-	def _on_identity_change(self):
-		"""Listen for patient *data* change."""
+	def _on_database_signal(self, **kwds):
+		# we don't have a patient: don't process signals
+		if isinstance(self.patient, gmNull.cNull):
+			return True
+
+		# we only care about identity and name changes
+		if kwds['table'] not in [u'dem.identity', u'dem.names']:
+			return True
+
+		# signal is not about our patient: ignore signal
+		if int(kwds['pk_identity']) != self.patient.ID:
+			return True
+
+		if kwds['table'] == u'dem.identity':
+			# we don't care about newly INSERTed or DELETEd patients
+			if kwds['operation'] != 'UPDATE':
+				return True
+
 		self.patient.refetch_payload()
+		return True
+
 	#--------------------------------------------------------
 	# external API
 	#--------------------------------------------------------
@@ -1909,14 +1935,13 @@ class gmCurrentPatient(gmBorg.cBorg):
 			raise TypeError(u'callback [%s] not callable' % callback)
 
 		self.__callbacks_before_switching_away_from_patient.append(callback)
+
 	#--------------------------------------------------------
 	def _get_connected(self):
 		return (not isinstance(self.patient, gmNull.cNull))
 
-	def _set_connected(self):
-		raise AttributeError(u'invalid to set <connected> state')
+	connected = property(_get_connected, lambda x:x)
 
-	connected = property(_get_connected, _set_connected)
 	#--------------------------------------------------------
 	def _get_locked(self):
 		return (self.__lock_depth > 0)
@@ -1927,18 +1952,20 @@ class gmCurrentPatient(gmBorg.cBorg):
 			gmDispatcher.send(signal = 'patient_locked', sender = self.__class__.__name__)
 		else:
 			if self.__lock_depth == 0:
-				_log.error('lock/unlock imbalance, trying to refcount lock depth below 0')
+				_log.error('lock/unlock imbalance, tried to refcount lock depth below 0')
 				return
 			else:
 				self.__lock_depth = self.__lock_depth - 1
 			gmDispatcher.send(signal = 'patient_unlocked', sender = self.__class__.__name__)
 
 	locked = property(_get_locked, _set_locked)
+
 	#--------------------------------------------------------
 	def force_unlock(self):
 		_log.info('forced patient unlock at lock depth [%s]' % self.__lock_depth)
 		self.__lock_depth = 0
 		gmDispatcher.send(signal = 'patient_unlocked', sender = self.__class__.__name__)
+
 	#--------------------------------------------------------
 	# patient change handling
 	#--------------------------------------------------------
@@ -1957,10 +1984,11 @@ class gmCurrentPatient(gmBorg.cBorg):
 				return False
 
 			if not successful:
-				_log.debug('callback [%s] returned False', call_back)
+				_log.error('callback [%s] returned False', call_back)
 				return False
 
 		return True
+
 	#--------------------------------------------------------
 	def __send_pre_unselection_notification(self):
 		"""Sends signal when current patient is about to be unset.
@@ -1973,6 +2001,7 @@ class gmCurrentPatient(gmBorg.cBorg):
 			'pk_identity': self.patient['pk_identity']
 		}
 		gmDispatcher.send(**kwargs)
+
 	#--------------------------------------------------------
 	def __send_unselection_notification(self):
 		"""Sends signal when the previously active patient has
@@ -1987,6 +2016,7 @@ class gmCurrentPatient(gmBorg.cBorg):
 			'sender': self.__class__.__name__
 		}
 		gmDispatcher.send(**kwargs)
+
 	#--------------------------------------------------------
 	def __send_selection_notification(self):
 		"""Sends signal when another patient has actually been made active."""
@@ -1996,14 +2026,24 @@ class gmCurrentPatient(gmBorg.cBorg):
 			'pk_identity': self.patient['pk_identity']
 		}
 		gmDispatcher.send(**kwargs)
+
 	#--------------------------------------------------------
 	# __getattr__ handling
 	#--------------------------------------------------------
 	def __getattr__(self, attribute):
+		# override __getattr__ here, not __getattribute__ because
+		# the former is used _after_ ordinary attribute lookup
+		# failed while the latter is applied _before_ ordinary
+		# lookup (and is easy to drive into infinite recursion),
+		# this is also why subsequent access to self.patient
+		# simply returns the .patient member value :-)
 		if attribute == 'patient':
 			raise AttributeError
-		if not isinstance(self.patient, gmNull.cNull):
-			return getattr(self.patient, attribute)
+		if isinstance(self.patient, gmNull.cNull):
+			_log.error("[%s]: cannot getattr(%s, '%s'), patient attribute not connected to a patient", self, self.patient, attribute)
+			raise AttributeError("[%s]: cannot getattr(%s, '%s'), patient attribute not connected to a patient" % (self, self.patient, attribute))
+		return getattr(self.patient, attribute)
+
 	#--------------------------------------------------------
 	# __get/setitem__ handling
 	#--------------------------------------------------------
@@ -2011,9 +2051,11 @@ class gmCurrentPatient(gmBorg.cBorg):
 		"""Return any attribute if known how to retrieve it by proxy.
 		"""
 		return self.patient[attribute]
+
 	#--------------------------------------------------------
 	def __setitem__(self, attribute, value):
 		self.patient[attribute] = value
+
 #============================================================
 # match providers
 #============================================================
@@ -2060,14 +2102,16 @@ INSERT INTO dem.names (
 ) VALUES (
 	currval('dem.identity_pk_seq'), coalesce(%s, 'xxxDEFAULTxxx'), coalesce(%s, 'xxxDEFAULTxxx')
 ) RETURNING id_identity"""
+#	cmd2 = u"select dem.add_name(currval('dem.identity_pk_seq')::integer, coalesce(%s, 'xxxDEFAULTxxx'), coalesce(%s, 'xxxDEFAULTxxx'), True)"
 	rows, idx = gmPG2.run_rw_queries (
 		queries = [
 			{'cmd': cmd1, 'args': [gender, dob]},
 			{'cmd': cmd2, 'args': [lastnames, firstnames]}
+			#{'cmd': cmd2, 'args': [firstnames, lastnames]}
 		],
 		return_data = True
 	)
-	ident = cIdentity(aPK_obj=rows[0][0])
+	ident = cIdentity(aPK_obj = rows[0][0])
 	gmHooks.run_hook_script(hook = u'post_person_creation')
 	return ident
 
@@ -2376,6 +2420,12 @@ if __name__ == '__main__':
 	def test_vcf():
 		person = cIdentity(aPK_obj = 12)
 		print person.export_as_vcard()
+
+	#--------------------------------------------------------
+	def test_current_patient():
+		pat = gmCurrentPatient()
+		print "pat.get_emr()", pat.get_emr()
+
 	#--------------------------------------------------------
 	#test_dto_person()
 	#test_identity()
@@ -2391,6 +2441,7 @@ if __name__ == '__main__':
 	#print "\n\nRetrieving communication media enum (id, description): %s" % comms
 	#test_export_area()
 	#test_ext_id()
-	test_vcf()
+	#test_vcf()
+	test_current_patient()
 
 #============================================================
